import pymxs
import os
import sys

from PySide2 import QtCore
from PySide2 import QtWidgets

# Constants
PARENT_DIR = os.path.dirname(os.path.abspath(__file__))
CONFIG_DIR_DIR = os.path.dirname(PARENT_DIR)

VERSION_STR = "1.2.0"
MENU_NAME = "GridMarkets"
LOGGER_NAME = 'gridmarkets_3ds_plugin'
PRODUCT = '3ds'
META_INFO = 'gridmarkets-3ds-plugin version: ' + VERSION_STR
DEBUG_MODE = os.path.exists(os.path.join(PARENT_DIR, 'debug'))

if sys.version_info[0] >= 3:
    import configparser
    config = configparser.ConfigParser()
    config.read(os.path.join(CONFIG_DIR_DIR, 'config.ini'))
    ENVOY_PYTHON_PATH = config['default']['ENVOY_PYTHON_PATH']
    SUBMIT2GM_PATH = config['default']['SUBMIT2GM_PATH']
else:
    import ConfigParser as configparser
    config = configparser.ConfigParser()
    config.read(os.path.join(CONFIG_DIR_DIR, 'config.ini'))
    ENVOY_PYTHON_PATH = config.get('default', 'ENVOY_PYTHON_PATH')
    SUBMIT2GM_PATH = config.get('default', 'SUBMIT2GM_PATH')


def evaluate_code(code, return_variables):
    # Execute the provided code
    try:
        with pymxs.mxstoken():
            exec (code)
    except Exception as e:
        raise e

    results = {}

    for variable in return_variables:
        results[variable] = eval(variable)

    return results


def process_events():
    try:
        from submit2gm.command_port_server_utils import process_events
        process_events()
    except ImportError:
        pass


def about_to_quit_slot():
    try:
        from submit2gm.command_port_server_utils import disconnect_from_submit2gm
        disconnect_from_submit2gm(ENVOY_PYTHON_PATH)
    except ImportError:
        pass


event_processor_timer = QtCore.QTimer()
event_processor_timer.timeout.connect(process_events)
event_processor_timer.start()

QtWidgets.QApplication.instance().aboutToQuit.connect(about_to_quit_slot)


def import_submit2gm():
    # Get the location of the submit2gm python package
    submit2gm_package_path = os.path.join(os.path.dirname(SUBMIT2GM_PATH), 'submit2gm')

    # Import the submit2gm package
    submit2gm_init_path = os.path.join(submit2gm_package_path, "__init__.py")

    if sys.version_info[0] >= 3:
        from importlib import util as importlib_util
        spec = importlib_util.spec_from_file_location('submit2gm', submit2gm_init_path)
        module = importlib_util.module_from_spec(spec)

        sys.modules[spec.name] = module
        spec.loader.exec_module(module)

    elif 'submit2gm' not in sys.modules:  # Check it has not already been added (only required for Python2)
        # Import the submit2gm.command_port_server_utils module
        submit2gm_command_port_server_utils_path = os.path.join(submit2gm_package_path, "command_port_server_utils.py")

        import imp
        module = imp.load_source('submit2gm', submit2gm_init_path)
        sys.modules['submit2gm'] = module

        module = imp.load_source('submit2gm.command_port_server_utils', submit2gm_command_port_server_utils_path)
        sys.modules['submit2gm.command_port_server_utils'] = module


def open_gridmarkets_plugin():
    import_submit2gm()
    from submit2gm.command_port_server_utils import start_command_port_server
    start_command_port_server(ENVOY_PYTHON_PATH, LOGGER_NAME, PRODUCT, META_INFO, evaluate_code, DEBUG_MODE)


def reload_plugin():
    global global_gm_3ds_plugin_module
    global_gm_3ds_plugin_module = reload(global_gm_3ds_plugin_module)
    global_gm_3ds_plugin_module.init(global_gm_3ds_plugin_module)


def is_cps_running():
    global global_command_port_server
    global global_cps_thread
    global global_submit2gm_thread

    if global_command_port_server:
        print("is cps running: " + str(global_command_port_server.isRunning()))

    if global_cps_thread:
        print("is cps thread alive: " + str(global_cps_thread.is_alive()))

    if global_submit2gm_thread:
        print("is submit2gm thread alive: " + str(global_submit2gm_thread.is_alive()))


def create_macroscript(_func, category="", name="", tool_tip="", button_text="", *args):
    try:
        # gets the qualified name for bound methods
        # ex: data_types.general_types.GMesh.center_pivot
        func_name = "{0}.{1}.{2}".format(
            _func.__module__, args[0].__class__.__name__, _func.__name__)
    except (IndexError, AttributeError):
        # gets the qualified name for unbound methods
        # ex: data_types.general_types.get_selection
        func_name = "{0}.{1}".format(
            _func.__module__, _func.__name__)

    script = """
    (
        python.Execute "import {}"
        python.Execute "{}()"
    )
    """.format(_func.__module__, func_name)
    return pymxs.runtime.macros.new(category, name, tool_tip, button_text, script)


GRIDMARKETS_CATEGORY = "GridMarkets"

SHOW_PLUGIN_ACTION_NAME = 'OpenGridMarketsPlugin'
SHOW_PLUGIN_ACTION_LABEL = 'Open GridMarkets Plugin'
SHOW_PLUGIN_MACRO = create_macroscript(open_gridmarkets_plugin,
                                       category=GRIDMARKETS_CATEGORY,
                                       name=SHOW_PLUGIN_ACTION_NAME,
                                       tool_tip="",
                                       button_text=SHOW_PLUGIN_ACTION_LABEL)
SHOW_PLUGIN_ACTION = pymxs.runtime.menuman.createActionItem(SHOW_PLUGIN_ACTION_NAME, GRIDMARKETS_CATEGORY)

RELOAD_PLUGIN_ACTION_NAME = 'ReloadGridMarketsPlugin'
RELOAD_PLUGIN_ACTION_LABEL = 'Reload GridMarkets Plugin'
RELOAD_PLUGIN_MACRO = create_macroscript(reload_plugin,
                                         category=GRIDMARKETS_CATEGORY,
                                         name=RELOAD_PLUGIN_ACTION_NAME,
                                         tool_tip="",
                                         button_text=RELOAD_PLUGIN_ACTION_LABEL)
RELOAD_PLUGIN_ACTION = pymxs.runtime.menuman.createActionItem(RELOAD_PLUGIN_ACTION_NAME, GRIDMARKETS_CATEGORY)

IS_CPS_RUNNING_NAME = 'IsCPSRunning'
IS_CPS_RUNNING_NAME_LABEL = 'Is CPS Running'
IS_CPS_RUNNING_MACRO = create_macroscript(is_cps_running,
                                         category=GRIDMARKETS_CATEGORY,
                                         name=IS_CPS_RUNNING_NAME,
                                         tool_tip="",
                                         button_text=IS_CPS_RUNNING_NAME_LABEL)
IS_CPS_RUNNING_ACTION = pymxs.runtime.menuman.createActionItem(IS_CPS_RUNNING_NAME, GRIDMARKETS_CATEGORY)


def create_menu(name):
    menu = pymxs.runtime.menuman.findmenu(MENU_NAME)
    if menu is None:
        menu = pymxs.runtime.menuMan.createMenu(name)

    menu.addItem(SHOW_PLUGIN_ACTION, -1)

    if DEBUG_MODE:
        separator = pymxs.runtime.menuMan.createSeparatorItem()
        menu.addItem(separator, -1)
        menu.addItem(RELOAD_PLUGIN_ACTION, -1)
        menu.addItem(IS_CPS_RUNNING_ACTION, -1)

    # Create a sub menu to contain the new menu
    sub_menu_item = pymxs.runtime.menuMan.createSubMenuItem(MENU_NAME, menu)

    # Add the new sub menu item to the main menu bar
    main_menu_bar = pymxs.runtime.menuMan.getMainMenuBar()
    main_menu_bar.addItem(sub_menu_item, -1)
    pymxs.runtime.menuman.updateMenuBar()


def create_main_window_and_menus():
    # Removing any previously left 'menu items'
    menu = pymxs.runtime.menuman.findmenu(MENU_NAME)
    if menu:
        pymxs.runtime.menuMan.unRegisterMenu(menu)

    # Create the GridMarkets menu item
    create_menu(MENU_NAME)


def init(gm_3ds_plugin_module):
    global global_gm_3ds_plugin_module
    global_gm_3ds_plugin_module = gm_3ds_plugin_module

    create_main_window_and_menus()
